# -*- coding: utf-8 -*-
# (c) 2015 Bright Interactive Limited. All rights reserved.
# http://www.bright-interactive.com | info@bright-interactive.com
import os
from unittest.case import TestCase
from mock import MagicMock, patch
from bright_fabric.tasks import pylint
from tests.util import ArgumentCaptor, OverrideFabricConfig


class TaskTests(TestCase):

    test_data_path = os.path.join('tests', 'test_data', 'tasks')

    py_in_path = [
        'one.py',
        os.path.join('subdir', 'two.py')
    ]

    def setUp(self):
        self.mock_fabric_api = MagicMock()

    @patch('bright_fabric.tasks.local')
    def test_pylint_runs_without_ignores_by_default(self, mock_local):
        pylint()
        captor = ArgumentCaptor()
        mock_local.assert_called_with(captor)
        self.assertTrue(all([
            captor.argument.startswith('flake8'),
            '--ignore' not in captor.argument,
        ]))

    @patch('bright_fabric.tasks.local')
    def test_pylint_runs_flake_with_ignores_if_configured(self, mock_local):
        with OverrideFabricConfig(pylint_ignore_errors=['E591', 'E100']):
            pylint()
            captor = ArgumentCaptor()
            mock_local.assert_called_with(captor)
            self.assertTrue(all([
                captor.argument.startswith('flake8'),
                '--ignore=E591,E100' in captor.argument,
            ]))

    @patch('bright_fabric.tasks.local')
    def test_pylint_runs_flake_on_all_python_files_in_configured_folders(self, mock_local):
        with OverrideFabricConfig(pylint_dirs=[self.test_data_path]):
            pylint()
            captor = ArgumentCaptor()
            mock_local.assert_called_with(captor)
            self.assertTrue(captor.argument.startswith('flake8'))
            for file in self.py_in_path:
                self.assertIn(file, captor.argument)

    @patch('bright_fabric.tasks.local')
    def test_pylint_runs_flake_on_current_working_path_if_no_dirs_configured(self, mock_local):
        pylint()
        captor = ArgumentCaptor()
        mock_local.assert_called_with(captor)
        self.assertTrue(captor.argument.startswith('flake8'))
        self.assertIn(os.path.abspath('fabfile.py'), captor.argument)

    @patch('bright_fabric.tasks.local')
    def test_pylint_ignores_excluded_dirs(self, mock_local):
        with OverrideFabricConfig(pylint_exclude_dirs=['subdir']):
            pylint()
            captor = ArgumentCaptor()
            mock_local.assert_called_with(captor)
            self.assertTrue(captor.argument.startswith('flake8'))
            self.assertNotIn(
                os.path.abspath(
                    os.path.join(
                        self.test_data_path, 'subdir', 'two.py'
                    )
                ),
                captor.argument
            )



from fabric.api import local, settings
from fabric.context_managers import hide
from fabric.state import env

from bright_fabric.util import abs_path, find_files, jslint_file


def pylint():
    """
    Validates the code layout for all python files in configured paths using flake8

    You can customise how this command is run by setting the following parameters
    in the fabric configuration (either through a config file or in env)

    pylint_ignore_errors: Set codes to ignore in a list (eg ['E500', 'E501'])
    pylint_dirs: Set dirs to search for python files in (defaults to current dir)
    pylint_exclude_dirs: Set dirs exclude when searching for python files to lint

    """
    flake8_command = 'flake8'

    ignore_errors = env.get('pylint_ignore_errors')

    if ignore_errors:
        flake8_command += ' --ignore=%s' % ','.join(ignore_errors)

    dirs = env.get('pylint_dirs', ['.'])

    exclude_dirs = env.get('pylint_exclude_dirs', [])

    all_files = []

    for dir in dirs:
        all_files += find_files(abs_path(dir), ['py'], exclude_dirs=exclude_dirs)

    all_files_for_cmd = "'" + "' '".join(all_files) + "'"
    with settings(hide('aborts', 'running')):
        local('%s %s' % (flake8_command, all_files_for_cmd))


def jslint():
    """
    We're using jslint-reporter:
        https://github.com/FND/jslint-reporter

    You'll need to install Node.js to run `fab jslint`:
        https://github.com/joyent/node/wiki/Installation

    To updgrade jslint use:
        `node tool/jslint-wrapper.js --upgrade`
    """

    with settings(warn_only=True):
        JS_ROOT = abs_path('static/js')
        SNIPPETS_ROOT = abs_path('templates/snippets')

        for filename in find_files(JS_ROOT, ['js'], exclude_dirs=['lib']):
            jslint_file(filename)
        # There are some JavaScript snippets in that get included into the
        # template HTML, so they have to be under templates/ not static/js/
        for filename in find_files(SNIPPETS_ROOT, ['js']):
            jslint_file(filename)



import os

from fabric.api import env, local
from fabric.utils import warn, puts


def find_files(path, suffixes=None, exclude_dirs=None):
    if not os.path.exists(path):
        return []

    exclude_dirs = exclude_dirs or []
    found_files = []

    def include_file(filename, suffixes):
        return suffixes is None or any([filename.endswith('.' + suffix) for suffix in suffixes])

    for root, dirs, files in os.walk(path):
        for exclude_dir in set(dirs) & set(exclude_dirs):
            dirs.remove(exclude_dir)
        found_files.extend([os.path.join(root, file) for file in files
                           if include_file(file, suffixes)])
    return found_files


def abs_path(relative_path, base_dir=env['cwd']):
    """
    Make relative paths absolute to the project directory.
    """
    return os.path.abspath(os.path.join(base_dir, relative_path))


def jslint_file(filename):
    JSLINT_CMD = (
        'node %s'
        ' --predef=jQuery,Modernizr,STATIC_URL,FileReader,plupload'
        ' --browser'
        ' --unparam'
        ' --regexp'
        ' --vars'
        ' --maxlen=120'
        ' --indent=4' % abs_path('tools/jslint-wrapper.js'))
    local('%s "%s"' % (JSLINT_CMD, filename))


def tag_matches_package_version(tag, application_version):

    if str(tag[1:]) != str(application_version):
        # using [1:] because of the tag form eg. v1.2.3
        warn("Package version (%s) does not match tag (%s)" % (application_version, tag))
        return False
    else:
        puts("Package version (%s) matches tag (%s)" % (application_version, tag))
        return True



# -*- coding: utf-8 -*-
# (c) 2015 Bright Interactive Limited. All rights reserved.
# http://www.bright-interactive.com | info@bright-interactive.com
from functools import partial
import os
from unittest.case import TestCase
from fabric.context_managers import hide
from bright_fabric.util import find_files, abs_path, tag_matches_package_version

suppress_output = partial(hide, 'everything')


class UtilTests(TestCase):

    test_data_path = os.path.join(os.path.dirname(__file__), 'test_data', 'util')
    txt_in_path = [
        'one.txt',
        'two.txt',
    ]

    dat_in_path = [
        'one.dat'
    ]

    txt_in_subdir = [
        os.path.join('subdir', 'three.txt')
    ]

    def test_find_files_finds_expected_in_folder_when_using_suffix(self):
        files = find_files(self.test_data_path, ['txt'])
        self.assertFilesReturned(files, self.txt_in_path + self.txt_in_subdir)

    def test_can_exclude_folders_from_find_files(self):
        files = find_files(self.test_data_path, ['txt'], exclude_dirs=['subdir'])
        self.assertFilesReturned(files, self.txt_in_path)

    def test_can_add_multiple_extensions_in_filter(self):
        files = find_files(self.test_data_path, ['txt', 'dat'], exclude_dirs=['subdir'])
        self.assertFilesReturned(files, self.txt_in_path + self.dat_in_path)

    def test_running_without_setting_suffixes_returns_all_suffixes(self):
        files = find_files(self.test_data_path, exclude_dirs=['subdir'])
        self.assertFilesReturned(files, self.txt_in_path + self.dat_in_path)

    def test_abs_path_gets_path_relative_to_absolute_path(self):
        self.assertEqual(os.path.join(os.path.abspath('basedir'), 'test'), abs_path('test', 'basedir'))

    def test_abs_path_defaults_to_current_directory_if_basedir_not_passed(self):
        self.assertEqual(os.path.join(os.path.abspath('.'), 'test'), abs_path('test'))

    def test_tag_matches_package_version(self):
        with suppress_output():
            self.assertTrue(tag_matches_package_version('v0.0.1', '0.0.1'))
            self.assertFalse(tag_matches_package_version('v0.0.2', '0.0.1'))

    def assertFilesReturned(self, files, expected_files):
        self.assertEqual(len(expected_files), len(files))
        for file_path in [os.path.join(self.test_data_path, file_name) for
                          file_name in self.txt_in_path]:
            self.assertIn(file_path, files)



# -*- coding: utf-8 -*-
# (c) 2015 Bright Interactive Limited. All rights reserved.
# http://www.bright-interactive.com | info@bright-interactive.com
from fabric.state import env


class ArgumentCaptor(object):

    def __eq__(self, argument):
        self.argument = argument
        return True


class OverrideFabricConfig(object):

    def __init__(self, **kwargs):
        self.initial_config = env.copy()
        self.overrides = env.copy()
        self.overrides.update(kwargs)

    def __enter__(self):
        env.clear()
        env.update(self.overrides)

    def __exit__(self, exc_type, exc_val, exc_tb):
        env.clear()
        env.update(self.initial_config)
